# encoding=utf-8
import datetime
import logging
import os
import re
import socket
import sys
import time
import platform

if platform.system() != 'Windows':
    from .timeroatingfilehandler import MultiProcessSafeHandler
else:
    from logging.handlers import TimedRotatingFileHandler as MultiProcessSafeHandler  # noqa


here = os.path.abspath(os.path.dirname(__file__))

log = logging.getLogger()


def tz_fix():
    # calculate TZ offset for isotime
    tz = re.compile(r'([+-]\d{2})(\d{2})$').match(time.strftime('%z'))
    if time.timezone and tz:
        opsym = '+' if time.timezone > 0 else '-'

        offset_hrs, offset_min = tz.groups()
        _tz_offset = "{0}{1}:{2}".format(opsym, offset_hrs, offset_min)
    else:
        # time.timezone == 0 => we're in UTC
        _tz_offset = "Z"
    return _tz_offset


class LogFormatter(logging.Formatter):
    """
    log formatter to add isotime, hostname
    """

    def __init__(self, service_name, *args, **kwargs):
        self._service_name = service_name
        self._tz_offset = tz_fix()

        # get hostname and save for later
        try:
            # force short-name
            self._hostname = socket.gethostname().split(".")[0]
        except Exception:
            self._hostname = "-"

        super(LogFormatter, self).__init__(*args, **kwargs)

    def format(self, record):
        """
        Add special content to record dict
        """
        record.hostname = self._hostname
        record.isotime = datetime.datetime.fromtimestamp(
            record.created).isoformat() + self._tz_offset
        record.service_name = self._service_name

        return logging.Formatter.format(self, record)


class RFC5424LogFormatter(LogFormatter):
    """
    formatter for rfc5424 messaging
    """

    RFC5424_LOG_FORMAT = (
        "%(isotime)s | %(hostname)s | %(process)-6s | %(service_name)s | "
        "%(levelname)-8s | %(name)s:%(filename)s:%(lineno)d | "
        "%(message)s"
    )

    RFC5424_LOG_FORMAT_THREAD = (
        "%(isotime)s | %(hostname)s | %(thread)d | %(threadName)s | "
        "%(levelname)-8s | %(name)s:%(filename)s:%(lineno)d | "
        "%(message)s"
    )

    RFC5424_TIME_FORMAT = None

    def __init__(self, service_name, thread=False):
        LogFormatter.__init__(
            self, service_name,
            fmt=self.RFC5424_LOG_FORMAT_THREAD if thread else self.RFC5424_LOG_FORMAT,
            datefmt=self.RFC5424_TIME_FORMAT)


def log_init(name, debug=None, log_path=None, when='MIDNIGHT', thread=False, **kwargs):
    """初始化配置logging

    :param debug: log message with level DEBUG or higher,
      add stdout handler to print log to screen
    :param log_path: log files path
    :return: log

    Usage::

        >>> from intelab_python_sdk.logger import log_init
        >>> # 初始化配置
        >>> log = log_init('test', debug=True, log_path='./logs')
        >>> log.info('info')
        2019-12-16T17:46:41.930061-+08:00 | wen-work-pc | 25689 | test | INFO    | root:<ipython-input-5-6a92ee17c096>:1 | info
        >>> log.debug('debug')
        2019-12-16T17:47:31.113047-+08:00 | wen-work-pc | 25689 | test | DEBUG   | root:<ipython-input-6-03b5a4252de9>:1 | debug
        >>> log.error('error')
        2019-12-16T17:48:02.338739-+08:00 | wen-work-pc | 25689 | test | ERROR   | root:<ipython-input-7-5bfd94e0c8ba>:1 | error
    """
    rfc5424_formatter = RFC5424LogFormatter(name, thread=thread)
    log.handlers = []

    stdout_handler = logging.StreamHandler(sys.stdout)
    stdout_handler.setFormatter(rfc5424_formatter)
    log.addHandler(stdout_handler)

    if (debug is None and log_path is None) or debug:
        log.setLevel(logging.DEBUG)
    else:
        log.setLevel(logging.INFO)
        if not debug and log_path is None:
            log.warning('log_path should not be null')

    if log_path:

        log_file = "{}/{}.info.log".format(log_path, name)
        file_log_handler = MultiProcessSafeHandler(
            log_file, when=when, **kwargs)
        file_log_handler.setFormatter(rfc5424_formatter)
        log.addHandler(file_log_handler)

        log_file = "{}/{}.error.log".format(log_path, name)
        file_log_handler_err = MultiProcessSafeHandler(
            log_file, when=when, **kwargs)
        file_log_handler_err.setFormatter(rfc5424_formatter)
        file_log_handler_err.setLevel(logging.ERROR)
        log.addHandler(file_log_handler_err)

    return log




from influxdb import InfluxDBClient
from . import ConnectError
from retrying import retry
from functools import wraps
from contextlib import contextmanager


def influxdb_query(influxdb_config: dict, types="r") -> any:
    """
    返回INFLUXDB连接和光标的装饰器, 使用方法如下：

    from negentropior.tools.connect.Influxdb import influxdb_query
    query_r = influxdb_query(mysql_config: dict, 'r')    >> 获取数据
    query_w = influxdb_query(mysql_config: dict, 'w')   >> 写入数据
    query_c = influxdb_query(mysql_config: dict)    >> 返回客户端对象

    @query_r
    def get_data_test(read, number):
        sql = f'''SELECT * FROM TABLE_NAME LIMIT {number}'''
        result = read(sql)
        result = result.raw['series'][0]['values']
        ......

    @query_w
    def insert_data(write, data: dataframe, measurement, tags=None, **kwargs):
        result = write(df, measurement='operating_status', tags={
                        "device_id": monitoring_target_id,
                        "device_name": name.strip(),
                        "device_type": type,
                        "sensor_name": sensor_name
                        }, retention_policy='utilizations')
        ...

    :param types:
    :param influxdb_config:
    :return:
    """

    if influxdb_config.get("HOST"):
        raise ConnectError("influxdb配置文件的命名都应该小写, 如host=..., 而不是HOST")

    def _load_func(function):

        @wraps(function)
        @retry(stop_max_attempt_number=5, wait_random_min=100, wait_random_max=1000)
        def _execute(*args, **kwargs):
            with _connect(influxdb_config) as client:
                if types == "w":
                    return function(client.write_points, *args, **kwargs)
                elif types == 'r':
                    return function(client.query, *args, **kwargs)
                else:
                    return function(client, *args, **kwargs)
        return _execute

    return _load_func


@contextmanager
def _connect(influxdb_config: dict) -> any:
    """
    建立和influxdb的连接并返回客户端
    :param influxdb_config:
    :return:
    """
    try:
        client = InfluxDBClient(**influxdb_config)
        yield client
    except ConnectError:
        raise ConnectError("influxdb连接错误, 请检查配置文件")
    except Exception as e:
        raise e






from retrying import retry
import json
from pika import BlockingConnection, URLParameters
from . import ConnectError


def producer(rabbitmq_config: dict, msg_list: list, queue_name: str) -> None:
    """
    rabbitmq生产者，注意msg_list为list, 其中包含的数据应该为dict
    使用方式如下：

    from negentropior.tools.connect.Rabbitmq import producer
    producer(rabbitmq_config, msg_list, queue_name)

    :param rabbitmq_config:
    :param msg_list:
    :param queue_name:
    :return:
    """
    connection = _connect(rabbitmq_config)
    channel = connection.channel()
    channel.queue_declare(queue=queue_name, durable=True)
    for msg in msg_list:
        if not isinstance(msg, dict):
            raise ConnectError("msg必须是dict")
        channel.basic_publish(
            exchange='',
            routing_key=queue_name,
            body=json.dumps(msg, ensure_ascii=False))
    channel.close()


def consumer(rabbitmq_config: dict, queue_name: str, log: any, alarm: any) -> any:
    """
    rabbitmq的消费者，注意log和alarm，log是用来记录日志的对象，alarm则是发送警告的函数对象
    使用方法如下：

    from negentropior.tools.connect.Rabbitmq import consumer
    rabbitmq_consumer = consumer(rabbitmq_config, queue_name, log, alarm)

    @rabbitmq_consumer
    def consume(*args, **kwargs):
        这里是对传来的mq_json, 具体消费的自定义相关操作
        ...

    :param rabbitmq_config:
    :param queue_name:
    :param log:
    :param alarm:
    :return:
    """
    connection = _connect(rabbitmq_config)
    channel = connection.channel()
    channel.queue_declare(queue=queue_name, durable=True)

    def _in_func(function):

        channel.basic_qos(prefetch_count=1)  # 类似权重，按能力分发，如果有一个消息，就不在给你发
        channel.basic_consume(  # 消费消息
            queue=queue_name,  # 要消费的队列
            on_message_callback=function  # 如果收到消息，就调用callback函数来处理消息
        )

        try:
            channel.start_consuming()
        except KeyboardInterrupt:
            log.info('MQ connection closed by user')
            pass
        except Exception as e:
            log.exception('MQ connection closed unexpectedly. %s', e)
            # TODO 通知到开发者
            alarm(e)
            raise

    return _in_func


@retry(stop_max_attempt_number=3, wait_random_min=100, wait_random_max=1000)
def _connect(rabbitmq_config: dict) -> any:

    connection = BlockingConnection(
        URLParameters('amqp://{user}:{password}@{host}:{port}/%2f?heartbeat={heartbeat}'.format(**rabbitmq_config)))

    return connection





import time
import dateutil.parser
from datetime import datetime, timedelta


def get_on_the_hour(times: any) -> datetime:
    """
    将输入时间的分秒舍去, 取整点
    :param times:
    :return:
    """
    if not isinstance(times, datetime):
        times = dateutil.parser.parse(times)
    return datetime(times.year, times.month, times.day, times.hour)


def datetime2str(times: datetime, _format='%Y-%m-%d %H:%M:%S') -> str:
    """
    将 datetime 格式的时间转化为 str 格式
    :param times:
    :param _format:
    :return:
    """
    return times.strftime(_format) if isinstance(times, datetime) else times


def str2datetime(times: str, _format='%Y-%m-%d %H:%M:%S') -> datetime:
    """
    将 str 格式的时间转化为 datetime 格式
    :param times:
    :param _format:
    :return:
    """
    return datetime.strptime(times, _format) if isinstance(times, str) else times


def str_utc2cst(times: str) -> str:
    """
    将 str 格式的 utc 时间转化为 str 格式的北京时间
    :param times:
    :return:
    """
    return datetime2str(str2datetime(times) + timedelta(hours=8))


def datetime2timestamp(times: datetime) -> int:
    """
    将 datetime 格式的时间转为 13 位毫秒时间戳
    :param times:
    :return:
    """
    return int(time.mktime(times.timetuple()) * 1000.0 + times.microsecond / 1000.0)


def timestamp2datetime(times: int) -> datetime:
    """
    将 13 位整数毫秒时间戳转化成 datetime 格式
    :param times:
    :return:
    """
    return datetime.fromtimestamp(times / 1000.0)




import os
import time
import codecs
import fcntl
from logging.handlers import TimedRotatingFileHandler


class Lock(object):

    def __init__(self, filename):
        self.filename = filename
        self.path, self.lock_file = filename.rsplit('/', 1)
        if not os.path.exists(self.path):
            os.makedirs(self.path)
        self.handle = open('{}/.{}'.format(self.path, self.lock_file), 'w')

    def acquire(self):
        fcntl.lockf(self.handle, fcntl.LOCK_EX)

    def release(self):
        fcntl.lockf(self.handle, fcntl.LOCK_UN)

    def __del__(self):
        try:
            self.handle.close()
        except Exception:
            pass


class MultiProcessSafeHandler(TimedRotatingFileHandler):
    def __init__(self, filename, when='h', interval=1,
                 backupCount=0, encoding=None, utc=False):
        TimedRotatingFileHandler.__init__(
            self, filename, when, interval, backupCount, encoding, True, utc)
        self.current_file_name = self.get_new_file_name()
        self.lock_file = Lock('{}.lock'.format(self.baseFilename))

    def shouldRollover(self, record):
        if self.current_file_name != self.get_new_file_name():
            return True
        return False

    def doRollover(self):
        if self.stream:
            self.stream.close()
            self.stream = None
        self.current_file_name = self.get_new_file_name()
        if self.backupCount > 0:
            for s in self.getFilesToDelete():
                os.remove(s)

    def get_new_file_name(self):
        return self.baseFilename + "." + time.strftime(self.suffix, time.localtime())

    def _open(self):
        if self.encoding is None:
            stream = open(self.current_file_name, self.mode)
        else:
            stream = codecs.open(self.current_file_name,
                                 self.mode, self.encoding)
        return stream

    def acquire(self):
        self.lock_file.acquire()

    def release(self):
        self.lock_file.release()




from requests import post
import json
import pandas as pd
from . import ConnectError
from pandas import DataFrame, Series


class TaoSClient:

    def __init__(self, td_config: dict, log: any):

        if td_config.get("HOST"):
            raise ConnectError("td配置文件的命名都应该小写, 如host=..., 而不是HOST")

        self.request_url = f"http://{td_config['host']}:{td_config['port']}/rest/sqlt/{td_config['db']}"
        self.auth = (td_config['user'], td_config['password'])
        self.time_out = (5, 5)
        self.log = log

    def query(self, query: str, measurement=None) -> any:
        """
        执行陶思的sql语句，存在查找字段时，则返回查找结果，查找字段为None，则进入创建表、删除数据、插入数据的操作，成功返回True
        使用方法如下：

        from negentropior.tools.connect.Tdengine import TaoSClient
        td = TaoSClient(td_config: dict, log: log对象)

        def get_sampling_data():
            sql = ''SELECT...'''
            result = td.query(sql, measurement"sampling_status")
            return result

        def insert_sampling_data():
            sql = '''INSERT...'''
            td.query(sql)

        def create_table():
            sql = '''CREATE...'''
            td.query(sql)

        def delete_sampling_data():
            sql = '''DELETE...'''
            td.query(sql)

        :param query: sql语句
        :param measurement: 查找字段
        :return:
        """
        data = None
        try:
            response = post(self.request_url, data=query, auth=self.auth, timeout=self.time_out)
            response_result = json.loads(response.text)

            if response.reason == 'OK':
                if measurement:
                    self.log.info(f'获取监控时序数据请求成功，请求参数正常, {query}')
                    data = response_result['data']
                    data = DataFrame(data, columns=['time_index', measurement])
                    data = Series(data[measurement].values, index=pd.to_datetime(data['time_index'], unit='ms'))
                else:
                    self.log.info(f'创建or写入or删除操作operating_status结果 {response_result}')
                    return True
            else:
                self.log.info(f'获取监控时序数据请求成功,请求参数异常, {query}')
        except Exception as e:
            self.log.error(f'获取监控时序数据请求报错, {e}')

        return data






# encoding=utf-8
import time
import urllib
from urllib import request
import base64
import hashlib
import hmac
import json
from urllib.error import HTTPError

try:
    import urllib.parse
    quote_plus = urllib.parse.quote_plus
except AttributeError:
    quote_plus = urllib.quote_plus

from ..log import log


def do_request(method, url, data=None, headers=None):
    if headers is None:
        headers = {'Content-Type': 'application/json'}
    if data is None:
        data = {}
    if method == 'GET' or method == 'DELETE':
        data = urllib.parse.urlencode(data)
        if data:
            url = url + '?' + data
        data = None
    else:
        data = json.dumps(data).encode('utf-8')

    req = request.Request(url, data, headers, method=method)
    try:
        response = request.urlopen(req)
        res_json = json.loads(response.read().decode('utf-8'))
    except HTTPError as e:
        log.error(e)
    except ValueError as e:
        log.error('JSONDecodeError.[{}]'.format(e))
    else:
        return res_json


class DingTalkMessage(object):
    """
    钉钉机器人消息推送
    Usage::
        >> from intelab_python_sdk.dingtalk import DingTalkMessage
        >> dingtalk = DingTalkMessage(webhook, secret)
        >> dingtalk.send_text('这是文本信息', mobiles=['15131601294'], at_all=False)  # mobiles是at群中的人(手机号),at_all为True是at全体
        >> dingtalk.send_link('这是标题', '这是内容', 'https://www.baidu.com')
        >> dingtalk.send_markdown('这是标题', '# 这是内容 \n> 这是引用\n [百度](https://baidu.com)')
    """

    headers = {"Content-Type": "application/json;charset=utf-8"}

    def __init__(self, webhook, secret=None, pc_slide=True):

        self.webhook = webhook
        self.pc_slide = pc_slide
        self.secret = secret
        self.pc_slide = pc_slide
        self.start_time = time.time()
        if self.secret is not None and self.secret.startswith('SEC'):
            self._update_webhook()

    def send_text(self, content, mobiles=None, at_all=False) -> any:
        """
        发送文本消息
        :param content: 文本内容
        :param mobiles: 群中at的用户，可以为空
        :param at_all: bool. at群中所有人，默认值为False
        """
        if mobiles is None:
            mobiles = []
        msg_dict = {
            "msgtype": "text",
            "text": {
                "content": content
            },
            "at": {
                "atMobiles": mobiles,
                "isAtAll": at_all
            }
        }
        return self._send(msg_dict)

    def send_link(self, title, text, message_url, pic_url='') -> any:
        """
        link类型
        :param title: 消息标题
        :param text: 消息内容（如果太长自动省略显示）
        :param message_url: 点击消息触发的URL
        :param pic_url: 图片URL（可选）
        :return: 返回消息发送结果
        """
        msg_dict = {
            "msgtype": "link",
            "link": {
                "text": text,
                "title": title,
                "picUrl": pic_url,
                "messageUrl": self._msg_open_type(message_url)
            }
        }
        log.debug('link类型：%s' % msg_dict)
        return self._send(msg_dict)

    def send_markdown(self, title, text, mobiles=None, at_all=False) -> any:
        """
        markdown
        :param title: 消息标题
        :param text: 消息内容。支持简单的md语法。
        :param mobiles:
        :param at_all:
        :return:
        """
        if mobiles is None:
            mobiles = []
        if mobiles:
            text += '\n@' + '@'.join(mobiles)

        msg_dict = {
            "msgtype": "markdown",
            "markdown": {
                "title": title,
                "text": text,
            },
            "at": {
                "atMobiles": mobiles,
                "isAtAll": at_all
            }
        }
        return self._send(msg_dict)

    def _update_webhook(self) -> any:
        """
        钉钉群自定义机器人安全设置加签时，签名中的时间戳与请求时不能超过一个小时，
        所以每个1小时需要更新签名
        """
        timestamp = round(self.start_time * 1000)
        string_to_sign = '{}\n{}'.format(timestamp, self.secret)
        hmac_code = hmac.new(self.secret.encode(), string_to_sign.encode(),
                             digestmod=hashlib.sha256).digest()

        sign = quote_plus(base64.b64encode(hmac_code))
        self.webhook = '{}&timestamp={}&sign={}'.format(
            self.webhook, str(timestamp), sign)

    def _msg_open_type(self, url) -> any:
        """
        消息链接的打开方式
        1、默认或不设置时，为浏览器打开：pc_slide=False
        2、在PC端侧边栏打开：pc_slide=True
        """
        return 'dingtalk://dingtalkclient/page/link?url={}&pc_slide={}'.format(
            quote_plus(url), 'true' if self.pc_slide else 'false')

    def _send(self, data) -> any:
        now = time.time()

        if now - self.start_time >= 3600 \
                and self.secret is not None \
                and self.secret.startswith('SEC'):
            self.start_time = now
            self._update_webhook()
        try:
            res = do_request('POST', url=self.webhook, data=data,
                             headers=self.headers)
            if 'errcode' in res and res['errcode'] == 0:
                log.info('dingtalk message sent is successful.')
                return True
            else:
                raise ValueError(res)
        except Exception as e:
            log.error("dingtalk message sent is unsuccessful!!!\n %s", e)
        return False



# encoding=utf-8
import requests
import json
import logging
import time
import urllib3
from requests_toolbelt import MultipartEncoder

urllib3.disable_warnings()

test_robot = ""
master_robot = ""

try:
    JSONDecodeError = json.decoder.JSONDecodeError
except AttributeError:
    JSONDecodeError = ValueError


def is_not_null_and_blank_str(content):
    """
    非空字符串
    :param content: 字符串
    :return: 非空 - True，空 - False
    """
    if content and content.strip():
        return True
    else:
        return False


def uploadImage():
    url = "https://open.feishu.cn/open-apis/im/v1/images"
    form = {'image_type': 'message',
            'image': (open('E:/test.png', 'rb'))}  # 需要替换具体的path
    multi_form = MultipartEncoder(form)
    # 获取tenant_access_token, 需要替换为实际的token
    headers = {'Authorization': 'Bearer t-xxx', 'Content-Type': multi_form.content_type}
    response = requests.request("POST", url, headers=headers, data=multi_form)
    print(response.headers['X-Tt-Logid'])  # for debug or oncall
    print(response.content)  # Print Response


class FeiShuTalkChatBot(object):

    def __init__(self, webhook, secret=None, pc_slide=False, fail_notice=False):
        """
        机器人初始化
        :param webhook: 飞书群自定义机器人webhook地址
        :param secret: 机器人安全设置页面勾选“加签”时需要传入的密钥
        :param pc_slide: 消息链接打开方式，默认False为浏览器打开，设置为True时为PC端侧边栏打开
        :param fail_notice: 消息发送失败提醒，默认为False不提醒，开发者可以根据返回的消息发送结果自行判断和处理
        """
        super(FeiShuTalkChatBot, self).__init__()
        self.headers = {'Content-Type': 'application/json; charset=utf-8'}
        self.webhook = webhook
        self.secret = secret
        self.pc_slide = pc_slide
        self.fail_notice = fail_notice

    def send_text(self, msg):
        """
        消息类型为text类型
        :param msg: 消息内容
        :return: 返回消息发送结果
        """
        data = {"msg_type": "text", "at": {}}
        if is_not_null_and_blank_str(msg):  # 传入msg非空
            data["content"] = {"text": msg}
        else:
            logging.error("text类型，消息内容不能为空！")
            raise ValueError("text类型，消息内容不能为空！")

        logging.debug('text类型：%s' % data)
        return self.post(data)

    def post(self, data):
        """
        发送消息（内容UTF-8编码）
        :param data: 消息数据（字典）
        :return: 返回消息发送结果
        """
        try:
            post_data = json.dumps(data)
            response = requests.post(self.webhook, headers=self.headers, data=post_data, verify=False)
        except requests.exceptions.HTTPError as exc:
            logging.error("消息发送失败， HTTP error: %d, reason: %s" % (exc.response.status_code, exc.response.reason))
            raise
        except requests.exceptions.ConnectionError:
            logging.error("消息发送失败，HTTP connection error!")
            raise
        except requests.exceptions.Timeout:
            logging.error("消息发送失败，Timeout error!")
            raise
        except requests.exceptions.RequestException:
            logging.error("消息发送失败, Request Exception!")
            raise
        else:
            try:
                result = response.json()
            except JSONDecodeError:
                logging.error("服务器响应异常，状态码：%s，响应内容：%s" % (response.status_code, response.text))
                return {'errcode': 500, 'errmsg': '服务器响应异常'}
            else:
                logging.debug('发送结果：%s' % result)
                # 消息发送失败提醒（errcode 不为 0，表示消息发送异常），默认不提醒，开发者可以根据返回的消息发送结果自行判断和处理
                if self.fail_notice and result.get('errcode', True):
                    time_now = time.strftime("%Y-%m-%d %H:%M:%S", time.localtime(time.time()))
                    error_data = {
                        "msgtype": "text",
                        "text": {
                            "content": "[注意-自动通知]飞书机器人消息发送失败，时间：%s，原因：%s，请及时跟进，谢谢!" % (
                                time_now, result['errmsg'] if result.get('errmsg', False) else '未知异常')
                        },
                        "at": {
                            "isAtAll": False
                        }
                    }
                    logging.error("消息发送失败，自动通知：%s" % error_data)
                    requests.post(self.webhook, headers=self.headers, data=json.dumps(error_data))
                return result


if __name__ == '__main__':
    # 以下为一个富文本发送至飞书群内的案例
    image_key = str(uploadImage())
    print(image_key)
    webhook = test_robot
    fei_shu = FeiShuTalkChatBot(webhook)
    rich_text = {
        "msg_type": "post",
        "content": {
            "post": {
                "zh_cn": {
                    "title": "日计于晨",
                    "content": [
                        [
                            {
                                "tag": "text",
                                "text": "Plan : "
                            },
                            {
                                "tag": "a",
                                "text": "click here",
                                "href": ""
                            },
                            {
                                "tag": "at",
                                "user_id": "all",
                                "user_name": "所有人"
                            }
                        ],
                        [
                            {
                                "tag": "text",
                                "text": "Study & Record : "
                            },
                            {
                                "tag": "a",
                                "text": "record here",
                                "href": ""
                            }
                        ],
                        [

                            {
                                "tag": "text",
                                "text": "Reading : "
                            },
                            {
                                "tag": "a",
                                "text": "share here",
                                "href": ""
                            }

                        ],
                        [
                            {
                                "tag": "img",
                                "image_key": image_key,
                                "width": 300,
                                "height": 300
                            }
                        ]
                    ]
                }
            }
        }
    }

    fei_shu.post(rich_text)




from flask import Flask

app = Flask(__name__)

"""
使用方法如下：
    >> 该方法用来制作接口给后端数据

    from negentropior.tools.Web.flaskAPI import app

    @app.route('/index': 页面地址, method = ['post'])
    def give_data():
        ...

    app.run(host=..., port=..., *args, **kwargs)
"""



import os
import oss2


class ALiYunOss:

    def __init__(self, oss_config: dict):
        """
        初始化配置并建立连接
        使用方式如下：

        from negentropior.tools.connect.AliyunOss import ALiYunOss as oss

        oss.upload_file()
        oss.isurl()
        oss.download_file()
        oss.batch_delete_files()

        :param oss_config:
        """
        self.config = oss_config
        self.region = self.config.get('region', 'hangzhou')
        self.local_endpoint = self.config.get('endpoint', f'https://oss-cn-{self.region}.aliyuncs.com')
        self.bucket = oss2.Bucket(oss2.Auth(self.config['access_key_id'], self.config['access_key_secret']),
                                  self.local_endpoint, self.config['bucket_name'])

    def upload_file(self, origin_file, local_file) -> any:
        """
        上传文件至oss
        :param origin_file:
        :param local_file:
        :return:
        """
        url_base = self.config.get('internal',
                                   f'https://{self.config["bucket_name"]}.oss-cn-{self.region}.aliyuncs.com/')
        result = self.bucket.put_object_from_file(origin_file, local_file)

        if result.status == 200:
            origin_file_url = os.path.join(url_base, origin_file)
        else:
            origin_file_url = ''
        return origin_file_url

    def isurl(self, origin_file):
        """
        check 文件是否存在oss中
        :param origin_file:
        :return:
        """
        file_url = self.bucket.sign_url('GET', origin_file, 10)
        return True if file_url else None

    def download_file(self, origin_file, local_file):
        """
        下载阿里云文件至本地
        :param local_file:
        :param origin_file:
        :return:
        """
        if origin_file.startswith('http://') or origin_file.startswith('https://'):
            origin_file = origin_file.split('/', 3)[-1]

        self.bucket.get_object_to_file(origin_file, local_file)
        return local_file

    def batch_delete_files(self, files_list):
        """单个或批量删除云端文件"""
        bucket_files = []
        for origin_file in files_list:
            if origin_file.startswith('http://') or origin_file.startswith('https://'):
                bucket_files.append(origin_file.split('/', 3)[-1])
            else:
                bucket_files.append(origin_file)

        result = self.bucket.batch_delete_objects(bucket_files)
        return result.deleted_keys



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from setuptools import setup, find_packages


with open("README.md", "r", encoding='utf-8') as fh:
    long_description = fh.read()

setup(name='negentropior',
      version='0.1.0',
      description='ILabor algorithm group tools 算法组常用工具方法',
      long_description=long_description,
      long_description_content_type="text/markdown",
      author='Yi OuYang, ZhiWen Wang',
      author_email='yiouyang143@gmail.com',
      packages=find_packages(),
      include_package_data=True,
      install_requires=[],
      classifiers=[
              "Programming Language :: Python :: 3",
              "License :: OSI Approved :: MIT License",
              "Operating System :: OS Independent",
          ],
)




from . import ConnectError
from contextlib import contextmanager
import pymysql as mysql
from retrying import retry
from functools import wraps


def mysql_query(mysql_config: dict) -> any:
    """
    返回MYSQL连接和光标的装饰器, 使用方法如下：

    from negentropior.tools.connect.mysql import mysql_query
    query = mysql_query(mysql_config: dict)

    @query
    def get_data_test(cursor, connection, number):
        sql = f'''SELECT * FROM TABLE_NAME LIMIT {number}'''
        ......

    :param mysql_config:
    :return:
    """
    if mysql_config.get("HOST"):
        raise ConnectError("mysql配置文件的命名都应该小写, 如host=..., 而不是HOST")

    def _load_func(function):

        @wraps(function)
        @retry(stop_max_attempt_number=5, wait_random_min=100, wait_random_max=1000)
        def _execute(*args, **kwargs):
            with _connect(mysql_config) as (cursor, connection):
                return function(cursor, connection, *args, **kwargs)
        return _execute

    return _load_func


@contextmanager
def _connect(mysql_config: dict) -> any:
    """
    建立和 mysql 的连接
    :param mysql_config:
    :return: mysql的连接和光标, 或者 error
    """
    connection = None

    try:
        connection = mysql.connect(**mysql_config)
        cursor = connection.cursor()
        yield cursor, connection
    except Exception as e:
        raise e
    finally:
        if connection:
            connection.close()



import pandas as pd
import csv
import os
from openpyxl.reader.excel import ExcelReader

# 须知：excel和csv对应在office类软件中的xlsx, xls表格文件


class Excel(ExcelReader):

    def __init__(self, path, sheet_index=0, sheet_name=None):
        """
        打开一个xlsx文件，并根据下标或名称选取sheet，进行相关初始化
        使用方法如下：

        self.worksheet 获取sheet, 为了节省代码, self.ws = self.worksheet
        >> 以下名称已经说明了用法，不再介绍
        self.ws.title
        self.ws.max_row
        self.ws.min_row
        self.ws.max_column
        self.ws.min_column
        self.ws.rows()
        self.ws.columns()
        self.ws.values()
        self.ws.iter_rows()
        self.ws.iter_columns()
        self.ws.append() >> 参数是iter类型(如list)的数据，按行输入, 如果想输入其它非iter类型的数据，可以考虑先本库的datatype转换数据类型
        >> from negentropior.tools.datatype import ...
        self.ws[] >> 该方法中括号中可以输入 '英文字母'、'数字'、'单元格名称' 来分别获取列对象、行对象、单元格的值

        :param path:
        :param sheet_index:
        :param sheet_name:
        """
        super(Excel, self).__init__(path, read_only=False, keep_vba=False, data_only=False, keep_links=True)
        self.read()

        self.sheet_names = self.wb.sheetnames
        self.encoding = self.wb.encoding
        self.properties = self.wb.properties
        self.remove = self.wb.remove
        self.create_excel = self.wb.create_sheet

        if not sheet_name:
            self.work_sheet = self.wb.worksheets[sheet_index]
        else:
            self.work_sheet = self.wb[sheet_name]

        self.ws = self.work_sheet


def get_excel_one_column(local_path, column_name: str) -> list:
    """
    获取 excel 其中一列并作为列表输出
    :param local_path:
    :param column_name: 列名
    :return:
    """
    df = pd.read_excel(local_path)
    column_list = df[column_name].values.tolist()
    return column_list


def csv_write_rows(local_path, data: list) -> None:
    """
    按行写入数据至 csv 文件中, csv 不存在则创建
    """
    f = open(local_path, "a", newline="", encoding='utf-8')
    writer = csv.writer(f)
    writer.writerows(data)
    f.close()


def all_excel2one(local_path, folders_path, sheet_name=0) -> list:
    """
    将文件夹内所有 excel 文件第 sheet_name 张表（默认为0）中的数据提取放入一个新的csv中
    :param local_path:
    :param folders_path:
    :param sheet_name:
    :return:
    """
    row_list = []
    for root, dirs, files in os.walk(folders_path):
        for file in files:
            try:
                read_data = pd.read_excel((folders_path + '/' + file), sheet_name=sheet_name)
                file_data = read_data.values.tolist()
                for row in file_data:
                    row_list.append(row)
            except Exception as e:
                print(e, type(e))
                continue
    csv_write_rows(local_path, row_list)
    return row_list


def xls2xlsm(local_path) -> any:
    """
    将 csv 文件转化为 excel 文件
    :param local_path:
    :return:
    """
    df = pd.read_excel(local_path)
    xlsm_excel = df.to_excel(local_path + 'm', index=False)
    return xlsm_excel


def excel2csv(local_path) -> any:
    """
    将 excel 文件转化为 csv 文件
    :param local_path:
    :return:
    """
    df = pd.read_excel(local_path)
    csv_data = df.to_csv(local_path[:-2], index=False)
    return csv_data

